# Extensions for KULTUR, especially the slideshow
package EPrints::Script::Compiled;

use strict;

sub run_kultur_slideshow
{
	my( $self, $state, $max ) = @_;

	if( $max->[1] ne "INTEGER" )
	{
		EPrints::abort( "Usage: kultur_slideshow( MAX )" );
	}

	$max = $max->[0];

	my $size = 'preview';

	my $session = $state->{session};

	my $docs = kultur_random_documents( $session, $max, $size );

	my $frag = $session->make_doc_fragment;

	my $div = $session->make_element( "div", id => "slideshow" );
	$frag->appendChild( $div );
	my $div_cnt = $session->make_element( "div", id => "slideshow_cnt" );
	$div->appendChild( $div_cnt );
	my $div_idx = $session->make_element( "div", id => "slideshow_idx", style => "display: none" );
	$div->appendChild( $div_idx );

	my $idx_ul = $session->make_element( "ul" );
	$div_idx->appendChild( $idx_ul );

	my $first = 1;
	foreach my $doc (@$docs)
	{
		my $eprint = $doc->get_eprint;

		# index icon
		my $li = $session->make_element( "li",
			class => "icon",
			title => $eprint->value( "title" ),
		);
		$idx_ul->appendChild( $li );
		$li->appendChild( $session->make_element( "img",
			src => $doc->icon_url( size => "small" )
			) );

		# slide
		my $div = $session->make_element( "div", class => "slide" );
		$div_cnt->appendChild( $div );
		if( $first )
		{
			$first = 0;
			$div->setAttribute( style => "display: block" );
		}
		else
		{
			$div->setAttribute( style => "display: none" );
			$li->setAttribute( style => "opacity: .5; filter: alpha(opacity=50); -moz-opacity: .5; -khtml-opacity: .5;" );
		}

		my $abs_link = $session->render_link( $doc->get_eprint->get_url );
		$div->appendChild( $abs_link );
		my $img = $session->make_element( "img",
			src => $doc->thumbnail_url( $size )
		);
		$abs_link->appendChild( $img );
		$div->appendChild( $session->make_element( "br" ) );

		my $caption = $eprint->render_citation_link( "default" );
		$div->appendChild( $caption );
	}

	return [ $frag, "XHTML" ];
}

sub kultur_random_documents
{
	my( $session, $max, $size ) = @_;

	my $docs = [];

	my $ids = _mysql_rand_docs( $session, $max*100 );

	my %seen;
	for(@$ids)
	{
		my( $eprintid, $docid ) = @$_;
		next if $seen{$eprintid};
		$seen{$eprintid} = 1;

		my $doc = EPrints::DataObj::Document->new( $session, $docid );
		next unless defined $doc;

		next unless $doc->thumbnail_url( $size );

		push @$docs, $doc;
		last if --$max == 0;
	}

	return $docs;
}

sub _mysql_rand_docs
{
	my( $session, $max ) = @_;

	my @ids;

	my $restrictions = $session->config( 'kultur', 'slideshow_restrictions' );	

	my $sql = "SELECT eprintid, document.docid, relation_type FROM eprint INNER JOIN document USING(eprintid) LEFT JOIN document_relation_type ON document.docid=document_relation_type.docid AND document_relation_type.relation_type='http://eprints.org/relation/isVolatileVersionOf' WHERE security='public' AND eprint_status='archive' AND document_relation_type.docid IS NULL";

	if( defined $restrictions && length $restrictions )
	{
		$sql .= " AND ( $restrictions  ) ";
	}

	$sql .= " ORDER BY RAND() LIMIT $max";
	
	my $sth = $session->get_database->prepare( $sql );
	$sth->execute;

	while(my @row = $sth->fetchrow_array)
	{
		push @ids, \@row;
	}

	return \@ids;
}

1;
